#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

#include "uTFT_ST7735.h"

/* Directii */
#define UP      10
#define DOWN    20
#define LEFT    30
#define RIGHT   40
#define NONE    50

/* Starile jocului */
#define START_SCREEN 101
#define IN_GAME      110
#define END_SCREEN   111

#define SNAKE_START_SIZE  10
#define SNAKE_MAX_SIZE    150
#define SNAKE_WIDTH       4
#define BUFSIZ            25
#define FOOD_LIN_OFFSET   15
#define FOOD_COL_OFFSET   10

const uint8_t snake_start_lin = _SCREEN_HEIGHT >> 1;
const uint8_t snake_start_col = ((_SCREEN_WIDTH >> 1) - SNAKE_START_SIZE) >> 1;
const uint8_t food_limit_lin = _SCREEN_HEIGHT - FOOD_LIN_OFFSET;
const uint8_t food_limit_col = _SCREEN_WIDTH - FOOD_COL_OFFSET;
const uint8_t block_offset = SNAKE_WIDTH >> 1;

struct cell {
    int16_t lin;
    int16_t col;
} pos_snake[SNAKE_MAX_SIZE], pos_food;

static long randx;
static char randf;

uint8_t current_state, prec_state, current_dir;
uint32_t current_size = SNAKE_START_SIZE;
uint8_t game_won, score;
char buff[BUFSIZ];

/* Functii pentru generarea de numere random (kind of :/ ) */
void my_srand(unsigned x) {
    randx = x;
    randf = 1;
}

uint32_t my_rand(void) {
    if (!randf) {
        my_srand(1);
    }

    return ((uint32_t)((randx = randx * 1103515245L + 12345) >> 16) & 077777);
}

/*
    Folosesc portul C pentru butoane, configuratia fiind:
    - stanga -> PC0
    - sus -> PC2
    - jos -> PC4
    - dreapta -> PC6
    - start/reset joc -> PD6
*/
void setup_buttons(void) {
    DDRC &= ~(1 << PC0);
    DDRC &= ~(1 << PC2);
    DDRC &= ~(1 << PC4);
    DDRC &= ~(1 << PC6);

    DDRD &= ~(1 << PD6);
    DDRD |= (1 << PD7);

    PORTC |= (1 << PC0) | (1 << PC2) | (1 << PC4) | (1 << PC6);
    PORTD |= (1 << PD6);
    PORTD &= ~(1 << PD7);
}

void check_buttons(void) {
    if (!(PINC & (1 << PC0))) {
        if (current_dir == RIGHT) return;
        current_dir = LEFT;
    } else if (!(PINC & (1 << PC2))) {
        if (current_dir == DOWN) return;
        current_dir = UP;
    } else if (!(PINC & (1 << PC4))) {
        if (current_dir == UP) return;
        current_dir = DOWN;
    } else if (!(PINC & (1 << PC6))) {
        if (current_dir == LEFT) return;
        current_dir = RIGHT;
    } else if (!(PIND & (1 << PD6))) {
        if (prec_state == NONE) {
            /* Dau start joc */
            prec_state = current_state;
        } else {
            /* Dau reset in timpul jocului sau dupa "Game over" */
            prec_state = END_SCREEN;
        }
        current_state = START_SCREEN;
    }
}

inline void print_start_screen(void) {
    fillScreen(Color565(255, 255, 255));
    setTextColor(Color565(0, 0, 0), Color565(255, 255, 255));
    setCursor(8, 75);
    print("Let the games begin!");
}

inline void print_end_screen(void) {
    fillScreen(Color565(0, 0, 0));
    setTextColor(Color565(255, 255, 255), Color565(0, 0, 0));
    setCursor(50, 75);

    if (game_won) {
        print("GG, WP!");
    } else {
        setCursor(38, 75);
        print("You lost :(");
    }

    setCursor(8, 85);
	print("Press BTN to replay!");
}

/* Generez mancarea pe o celula libera din matrice */
inline void generate_food(void) {
    uint8_t lin;
    uint8_t col;
    uint8_t i, found = 0;

    while (!found) {
        found = 1;
        lin = (my_rand() % food_limit_lin) + FOOD_LIN_OFFSET;
        col = (my_rand() % food_limit_col) + FOOD_COL_OFFSET;

        for (i = 0; i < current_size; ++i) {
            if (pos_snake[i].lin == lin && pos_snake[i].col == col) {
                found = 0;
            }
        }
    }

    pos_food.lin = lin;
    pos_food.col = col;
}

inline void setup_snake(void) {
    uint16_t i, cnt = (SNAKE_START_SIZE - 1) * SNAKE_WIDTH;

    current_dir = RIGHT;
    current_size = SNAKE_START_SIZE;

    for (i = 0; i < current_size; ++i, cnt -= SNAKE_WIDTH) {
        pos_snake[i].col = snake_start_col + cnt;
        pos_snake[i].lin = snake_start_lin;
    }

    generate_food();
    fillRect(pos_food.col - block_offset, pos_food.lin - block_offset,
             SNAKE_WIDTH, SNAKE_WIDTH, Color565(255, 0, 0));
}

void get_new_head(struct cell *new_head) {
    if (current_dir == UP) {
        new_head->lin -= SNAKE_WIDTH;
    } else if (current_dir == DOWN) {
        new_head->lin += SNAKE_WIDTH;
    } else if (current_dir == LEFT) {
        new_head->col -= SNAKE_WIDTH;
    } else if (current_dir == RIGHT) {
        new_head->col += SNAKE_WIDTH;
    }

    if (new_head->lin > _SCREEN_HEIGHT) {
        new_head->lin = 0;
    } else if (new_head->lin < 0) {
        new_head->lin = _SCREEN_HEIGHT;
    }

    if (new_head->col > _SCREEN_WIDTH) {
        new_head->col = 0;
    } else if (new_head->col < 0) {
        new_head->col = _SCREEN_WIDTH;
    }
}

void print_map(void) {
    uint32_t i;
    struct cell new_head = pos_snake[0];

    get_new_head(&new_head);

    /* Verific daca m-am ciocnit cu mine insumi */
    for (i = 0; i < current_size; ++i) {
        if (pos_snake[i].lin == new_head.lin 
            && pos_snake[i].col == new_head.col) {

            prec_state = current_state;
            current_state = END_SCREEN;
            return;
        }
    }

    /* Daca noua pozitie se suprapune peste mancare */
    if (abs(new_head.lin - pos_food.lin) < SNAKE_WIDTH 
        && abs(new_head.col - pos_food.col) < SNAKE_WIDTH) {
        /* Sterg mancarea */
        fillRect(pos_food.col - block_offset, pos_food.lin - block_offset, 
                 SNAKE_WIDTH, SNAKE_WIDTH, Color565(255, 255, 255));

        /* Generez mancare noua */
        generate_food();

        PORTD |= (1 << PD7);
        _delay_ms(30);
        PORTD &= ~(1 << PD7);

        /* Cresc dimensiunea sarpelui si scorul */
        ++current_size;
        ++score;
    } else {
        /* Sterg ultima bucata din sarpe */
        fillRect(pos_snake[current_size - 1].col - block_offset, 
                 pos_snake[current_size - 1].lin - block_offset, 
                 SNAKE_WIDTH, SNAKE_WIDTH, Color565(255, 255, 255));
    }

    if (score >= SNAKE_MAX_SIZE) {
        current_state = END_SCREEN;
        game_won = 1;
        return;
    }

    /* Shiftez sharpele */
    for (i = current_size - 1; i > 0; --i) {
        pos_snake[i] = pos_snake[i - 1];
    }

    /* Adaug noul cap */
    pos_snake[0] = new_head;
	
    /* Afisez scorul */
	sprintf(buff, "Score: %d", score);
	setTextColor(Color565(205,106,0), Color565(255, 255, 255));
    setCursor(2, 2);
    print(buff);

    /* Afisez noua pozitie a capului */
    fillRect(new_head.col - block_offset, new_head.lin - block_offset,
             SNAKE_WIDTH, SNAKE_WIDTH, Color565(0, 0, 0));

    /* Afisez mancarea */
    fillRect(pos_food.col - block_offset, pos_food.lin - block_offset,
             SNAKE_WIDTH, SNAKE_WIDTH, Color565(255, 0, 0));
}

inline void reset_game(void) {
    prec_state = START_SCREEN;
    current_state = IN_GAME;
    current_dir = RIGHT;
    game_won = 0;
    score = 0;

    setup_snake();
    fillScreen(Color565(255, 255, 255));
}

int main(void) {
    setup_buttons();
    init_lcd();

    prec_state = NONE;
    current_state = START_SCREEN;	
	setup_snake();

    while (1) {
        check_buttons();

        if (prec_state == NONE && current_state == START_SCREEN) {
            print_start_screen();
            _delay_ms(200);
            continue;
        }

        if (prec_state == END_SCREEN && current_state == START_SCREEN) {
            reset_game();
            continue;
        }

        if (current_state == IN_GAME) {
            print_map();
            _delay_ms(105);
            continue;
        }

        if (current_state == END_SCREEN) {
            print_end_screen();
            _delay_ms(300);
        }
    }
    return 0;
}
